#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# Example of working with several devices simultaneously

from terasense import processor as tp

# Detect available devices
enumerator = tp.deviceEnumerator()
deviceList=enumerator.getDevList()
print("Connected devices:", deviceList)


#Initialize devices using serial numbers
sources = [tp.processor(sn=devID) for devID in deviceList]

#Start acquisition for every device
for dev in sources:
    dev.start()

#Read devices
for dev in sources:
    # do something with data
    print("id:",dev.interface.get_id(), ", max value:", dev.read().max())

# Call stop() to stop acquisition and join the acquisition and processing thread
for dev in sources:
    dev.stop()


# After initializing devices, the list of available devices should be empty
enumerator.update()
print("Free devices left:",enumerator.getDevList())


