import wx
import random, math

# GUI Video panel class
class VideoPanel(wx.Panel):
    """"Class VideoPanel"""
    def __init__(self,parent,id, main = None):

        super(VideoPanel, self).__init__(parent=parent,id=id)

        if not main:
            self.main = parent.GetParent()
        else:
            self.main = main
        
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)

        

        vbox.Add(wx.StaticText(self,label="Zoom")) 
        self.zoom = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS, minValue=0, maxValue=5, value=self.GetZoom())
        self.zoom.SetTickFreq(1)
        self.Bind(wx.EVT_SLIDER, self.OnChangeZoomDistance, id=self.zoom.GetId())
        vbox.Add( self.zoom, proportion=1, flag=wx.EXPAND )
        
        self.distanceLabel = wx.StaticText(self,label="Distance")
        vbox.Add(self.distanceLabel) 
        self.distance = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS, value=self.GetDistance())
        self.distance.SetTickFreq(10)
        self.Bind(wx.EVT_SLIDER, self.OnChangeZoomDistance, id=self.distance.GetId())
        vbox.Add( self.distance, proportion=1, flag=wx.EXPAND )

       
        vbox.Add(wx.StaticText(self,label="Brightness"))
        self.videobrightness = wx.Slider(self, style=wx.SL_INVERSE | wx.SL_TOP |wx.SL_AUTOTICKS, value=50)
        self.videobrightness.SetTickFreq(10)
        self.Bind(wx.EVT_SLIDER, self.OnChangeVideoBrightness, id=self.videobrightness.GetId())
        vbox.Add( self.videobrightness, proportion=1, flag=wx.EXPAND )

        
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.videobw = wx.CheckBox(self, label="BW video", name="videobw")
        self.Bind(wx.EVT_CHECKBOX, self.OnVideoBW, id=self.videobw.GetId())
        self.videobw.SetValue(False)
        hbox.Add( self.videobw, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)

        self.drawFrame = wx.CheckBox(self, label="Draw frame", name="drawFrame")
        self.Bind(wx.EVT_CHECKBOX, self.OnDrawFrame, id=self.drawFrame.GetId())
        self.drawFrame.SetValue(True)
        hbox.Add( self.drawFrame, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)

        self.drawGrid = wx.CheckBox(self, label="Draw grid", name="drawGrid")
        self.Bind(wx.EVT_CHECKBOX, self.OnDrawFrame, id=self.drawGrid.GetId())
        self.drawGrid.SetValue(False)
        hbox.Add( self.drawGrid, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)

        vbox.Add( hbox,proportion=1, flag=wx.EXPAND )

        
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND )

        
        self.SetSizerAndFit(vbox)

    def Initialize(self):
        self.OnChangeZoomDistance()
        self.OnChangeVideoBrightness()
        self.OnVideoBW()
        self.OnDrawFrame()

    def OnStart(self):
        self.Initialize()

    def OnStop(self):
        pass



    def GetZoom(self,event=None):
        if (self.main.worker):
            return int(round((self.main.worker.zoom-1)*5))
        else:
            return 0

   
    def GetDistanceString(self,d):
        if (d > 0 and d < 100000):            
            return "Distance: %.2f m" % (d/1000)
        elif (d >= 100000):
            return "Distance: infinity"
        else:
            return "Distance"
        
    def GetDistance(self):
        if (self.main.worker):            
            if (self.main.worker.distance > 2*self.main.worker.focus):
                return int(math.atan((float(self.main.worker.distance)/self.main.worker.focus/6 - 1./3))/math.pi*200)
            else:
                return 0
        else:
            return 0

    def OnChangeZoomDistance(self,event=None):
        if (self.main.worker):
            valzoom = float(self.zoom.GetValue())/5+1
            valdistance = self.main.worker.focus * 6*(1./3 + math.tan(float(self.distance.GetValue())*math.pi/200))
            self.main.worker.SetZoomAndDistance(valzoom,valdistance)
            self.distanceLabel.SetLabel(self.GetDistanceString(self.main.worker.distance))


    def OnChangeVideoBrightness(self,event=None):
        if (self.main.worker):
            val = self.videobrightness.GetValue()
            self.main.worker.videobrightness = float(val)/100

    def OnVideoBW(self,event=None):
        if self.main.worker:
            if self.videobw.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.VIDEOBW
            else :
                self.main.worker.processFlags &= ~self.main.worker.VIDEOBW

    def OnDrawFrame(self,event=None):
        if self.main.worker:
            if self.drawFrame.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.DRAW_FRAME
            else :
                self.main.worker.processFlags &= ~self.main.worker.DRAW_FRAME
            if self.drawGrid.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.DRAW_GRID
            else :
                self.main.worker.processFlags &= ~self.main.worker.DRAW_GRID
