import wx
import  wx.lib.intctrl
import random, math

# Default level of black
DEFAULT_BLACK=10

# GUI ROI panel class
class CoordinateDialog(wx.Dialog):
    """ """
    def __init__(self, parent, value=None, limits=None,  *args, **kw):
        super(CoordinateDialog, self).__init__(parent,*args, **kw)

        if isinstance(limits,tuple) and value is None:
            self.value = limits
        elif value is None:
            self.value = ((0,0), (0,0))
        else:
            self.value = value

#        panel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add((10,10),proportion=1)
        
        point1 = wx.StaticBox(self, label='Top Left')
        sizer = wx.StaticBoxSizer(point1, orient=wx.HORIZONTAL)
        sizer.Add(wx.StaticText(self, label="X:"),flag=wx.ALIGN_CENTER)
        if isinstance(limits,tuple):
            self.p1x =  wx.lib.intctrl.IntCtrl (self,value=self.value[0][0],min=limits[0][0], max=limits[1][0],limited = True)
        else:
            self.p1x =  wx.lib.intctrl.IntCtrl (self,value=self.value[0][0])
        sizer.Add(self.p1x,flag=wx.ALIGN_CENTER)
        sizer.Add((10,0),proportion=1)
        sizer.Add(wx.StaticText(self, label="Y:"),flag=wx.ALIGN_CENTER)
        if isinstance(limits,tuple):
            self.p1y =  wx.lib.intctrl.IntCtrl (self,value=self.value[0][1],min=limits[0][1], max=limits[1][1],limited = True)
        else:
            self.p1y =  wx.lib.intctrl.IntCtrl (self,value=self.value[0][1])
        sizer.Add(self.p1y,flag=wx.ALIGN_CENTER)

        vbox.Add(sizer)
        vbox.Add((10,10),proportion=1)

        point2 = wx.StaticBox(self, label='Bottom Right')
        sizer = wx.StaticBoxSizer(point2, orient=wx.HORIZONTAL)
        sizer.Add(wx.StaticText(self, label="X:"),flag=wx.ALIGN_CENTER)
        if isinstance(limits,tuple):
            self.p2x = wx.lib.intctrl.IntCtrl(self,value=self.value[1][0],min=limits[0][0], max=limits[1][0],limited = True)
        else:
            self.p2x =  wx.lib.intctrl.IntCtrl(self,value=self.value[1][0])
        sizer.Add(self.p2x,flag=wx.ALIGN_CENTER)
        sizer.Add((10,0),proportion=1)
        sizer.Add(wx.StaticText(self, label="Y:"),flag=wx.ALIGN_CENTER)
        if isinstance(limits,tuple):
            self.p2y =  wx.lib.intctrl.IntCtrl(self,value=self.value[1][1],min=limits[0][1], max=limits[1][1],limited = True)
        else:
            self.p2y =  wx.lib.intctrl.IntCtrl(self,value=self.value[1][1])
        sizer.Add(self.p2y,flag=wx.ALIGN_CENTER)

        vbox.Add(sizer)
        vbox.Add((10,10),proportion=1)


        hbox = wx.BoxSizer(wx.HORIZONTAL)
        okButton = wx.Button(self, label='Ok')
        closeButton = wx.Button(self, label='Cancel')
        hbox.Add(okButton)
        hbox.Add(closeButton, flag=wx.LEFT, border=5)

        
        vbox.Add(hbox, 
            flag=wx.ALIGN_CENTER|wx.TOP|wx.BOTTOM, border=10)

        self.SetSizerAndFit(vbox)
        
        okButton.Bind(wx.EVT_BUTTON, self.OnOK)
        closeButton.Bind(wx.EVT_BUTTON, self.OnClose)

    def OnOK(self, e):
        self.value = ( ( self.p1x.GetValue(), self.p1y.GetValue()), (self.p2x.GetValue(), self.p2y.GetValue()) )
        self.EndModal(True)

    def OnClose(self, e):
        self.EndModal(False)

    def GetValue(self):
        return ((min(self.value[0][0],self.value[1][0]),min(self.value[0][1],self.value[1][1])), (max(self.value[0][0],self.value[1][0]),max(self.value[0][1],self.value[1][1])))
    
class WorkingAreaPanel(wx.Panel):
    """"Class BCGPanel"""
    def __init__(self,parent,id, main = None, processor=None):

        super(WorkingAreaPanel, self).__init__(parent=parent,id=id)

        if not main:
            self.main = parent.GetParent()
        else:
            self.main = main

        self.p1 = (0,0)
        self.p2 = (self.main.imgSize[0]-1,self.main.imgSize[1]-1)
        
        vbox = wx.BoxSizer(wx.VERTICAL)
#        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)



        displayBox = wx.StaticBox(self, label = "Working area")
        self.displayText = wx.StaticText(self, label="%2d:%2d, %2d:%2d" % (self.p1[0],self.p2[0],self.p1[1],self.p2[1]))
        font = self.displayText.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        font.SetPointSize(int(font.GetPointSize()*2))
        self.displayText.SetFont(font)
        self.displayText.SetMinSize(self.displayText.GetTextExtent("000:000, 000:000"))    
        hbox = wx.StaticBoxSizer(displayBox, wx.VERTICAL)
        hbox.Add( (0,7),proportion=1, flag=wx.EXPAND)
        hbox.Add(self.displayText,proportion=0,flag=wx.ALIGN_CENTER|wx.RIGHT|wx.LEFT,border=5)
        hbox.Add( (0,10),proportion=1, flag=wx.EXPAND)
        vbox.Add( hbox,proportion=0, flag=wx.EXPAND )
        

        self.change = wx.Button(self,-1, 'Change...')
        self.Bind(wx.EVT_BUTTON, self.OnChange,id=self.change.GetId())
        vbox.Add(self.change, proportion = 0,flag=wx.EXPAND )
        
        self.toselect = wx.Button(self,-1, 'Set to Selection')
        self.Bind(wx.EVT_BUTTON, self.OnToSelect,id=self.toselect.GetId())
        vbox.Add(self.toselect, proportion = 0,flag=wx.EXPAND )
        
        self.reset = wx.Button(self,-1, 'Reset to full')
        self.Bind(wx.EVT_BUTTON, self.OnReset,id=self.reset.GetId())
        vbox.Add(self.reset, proportion = 0,flag=wx.EXPAND )
                
        vbox.Add( (0,5),proportion=0, flag=wx.EXPAND )
        self.hint = wx.StaticText(self, label="",style=wx.ALIGN_CENTRE    )
##        self.hint = wx.TextCtrl(self, style=wx.TE_MULTILINE| wx.TE_READONLY|wx.TE_CENTRE)
        vbox.Add(self.hint, proportion = 0,flag=wx.EXPAND )
        
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND )

        self.SetSizerAndFit(vbox)
        self.hint.SetLabel("Please, stop acquisition\n to change the working area.")

        
        self.hint.Hide()
        

    def Initialize(self):
        pass

    def OnStart(self):
        self.Initialize()
        self.change.Disable()
        self.toselect.Disable()
        self.reset.Disable()
        

        self.hint.Show()
        self.Layout()


    def OnStop(self):
        self.change.Enable()
        self.toselect.Enable()
        self.reset.Enable()
        self.hint.Hide()
  


    def OnChange(self, event=None):
        p1 = self.p1
        selection = self.main.image.GetSelection()
        dialog = CoordinateDialog(None,value = (self.p1, self.p2),limits = ((0,0), (self.main.imgSize[0]-1,self.main.imgSize[1]-1)), 
            title='Change Working Area')        
        if dialog.ShowModal():
            self.ChangeWA(*dialog.GetValue())
            if selection[0] is not None:
                self.main.image.SetSelection((p1[0]+selection[0][0]-self.p1[0], p1[1]+selection[0][1]-self.p1[1]), (p1[0]+selection[1][0]-self.p1[0],p1[1]+selection[1][1]-self.p1[1]))
        dialog.Destroy()        


    def OnToSelect(self, event=None):
        selection = self.main.image.GetSelection()
        if isinstance(selection,tuple) and isinstance(selection[0],tuple) and isinstance(selection[1],tuple):
            self.ChangeWA( (self.p1[0]+selection[0][0], self.p1[1]+selection[0][1]), (self.p1[0]+selection[1][0],self.p1[1]+selection[1][1]) )
            self.main.image.SelectAll()
            
        
    def OnReset(self, event=None):
        p1 = self.p1
        selection = self.main.image.GetSelection()
        self.ChangeWA( (0,0), (self.main.imgSize[0]-1, self.main.imgSize[1]-1) )
        if selection[0] is not None:
            self.main.image.SetSelection((p1[0]+selection[0][0], p1[1]+selection[0][1]), (p1[0]+selection[1][0],p1[1]+selection[1][1]))
        

    def ChangeWA(self,p1,p2):
        try:
            self.p1 = ( max(0, min(p1[0],p2[0])), max(0, min(p1[1],p2[1])))
            self.p2 = ( min(self.main.imgSize[0]-1, max(p1[0],p2[0])),  min(self.main.imgSize[1]-1, max(p1[1],p2[1])) )
##            self.p1 = ( max(0, min(p1[1],p2[1])), max(0, min(p1[0],p2[0])) )
##            self.p2 = ( min(self.main.imgSize[1]-1, max(p1[1],p2[1])), min(self.main.imgSize[0]-1, max(p1[0],p2[0])) )
            self.displayText.SetLabel("%3d:%3d, %3d:%3d" % (self.p1[0],self.p2[0],self.p1[1],self.p2[1]))
            self.main.SetWorkingArea(  (self.p1,  self.p2) )
            
        except:
            raise
    
